@@ -4,6 +4,10 @@ from pma_python import core, pma
 
 __version__ = pma.__version__
 
+_pma_casecollections_json = {}
+_pma_projects_json = {} 
+_pma_session_json = {}
+
 def get_version_info(pmacontrolURL):
 	"""
 	Get version info from PMA.control instance running at pmacontrolURL
@@ -55,15 +59,54 @@ def _pma_get_case_collections(pmacontrolURL, pmacoreSessionID):
 	Retrieve all the data for all the defined case collections in PMA.control
 	(RAW JSON data; not suited for human consumption)
 	"""
+	global _pma_casecollections_json
+	
 	url = pma._pma_join(pmacontrolURL, "api/CaseCollections?sessionID=" + pma._pma_q(pmacoreSessionID))
-	print (url)
+	if not (url in _pma_casecollections_json):
+		print (url)
+		try:
+			headers = {'Accept': 'application/json'}
+			r = requests.get(url, headers=headers)
+			_pma_casecollections_json[url] = r.json()
+		except Exception as e:
+			return None		
+
+	return _pma_casecollections_json[url]
+
+def get_case_collection_titles(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
+	"""
+	Retrieve case collections (possibly filtered by project ID), titles only
+	"""
 	try:
-		headers = {'Accept': 'application/json'}
-		r = requests.get(url, headers=headers)
+		return list(get_case_collection_titles_dict(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID).values())
 	except Exception as e:
 		return None		
-	return r.json()
+	
+def get_case_collection_titles_dict(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
+	"""
+	Retrieve case collections (possibly filtered by project ID), return a dictionary of case collection IDs and titles
+	"""
+	dct = {}
+	all_colls = _pma_get_case_collections(pmacontrolURL, pmacoreSessionID)
+	for coll in all_colls:
+		if pmacontrolProjectID == None:
+			dct[coll["Id"]] = coll["Title"]
+		elif pmacontrolProjectID == coll["ModuleId"]:
+			dct[coll["Id"]] = coll["Title"]
 
+	return dct
+
+def get_case_collection(pmacontrolURL, pmacontrolCaseCollectionID, pmacoreSessionID):
+	"""
+	Retrieve case collection details
+	"""
+	all_colls = _pma_get_case_collections(pmacontrolURL, pmacoreSessionID)
+	for coll in all_colls:
+		if coll["Id"] == pmacontrolCaseCollectionID:
+			return coll
+
+	return None
+	
 def _pma_format_project_embedded_sessions_properly(original_project_sessions):
 	"""
 	Helper method to convert a list of sessions with default arguments into a summarized dictionary
@@ -79,15 +122,19 @@ def _pma_get_projects(pmacontrolURL, pmacoreSessionID):
 	Retrieve all projects and their data in PMA.control 
 	(RAW JSON data; not suited for human consumption)
 	"""
+	global _pma_projects_json
+	
 	url = pma._pma_join(pmacontrolURL, "api/Projects?sessionID=" + pma._pma_q(pmacoreSessionID))
-	print (url)
-	try:
-		headers = {'Accept': 'application/json'}
-		r = requests.get(url, headers=headers)
-	except Exception as e:
-		return None		
-	return r.json()
-
+	if not (url in _pma_projects_json):
+		print (url)
+		try:
+			headers = {'Accept': 'application/json'}
+			r = requests.get(url, headers=headers)
+		except Exception as e:
+			return None		
+		_pma_projects_json[url] = r.json()
+		
+	return _pma_projects_json[url]
 
 def get_project_titles(pmacontrolURL, pmacoreSessionID):
 	"""
@@ -133,6 +180,29 @@ def get_project(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
 		
 	return None
 
+def get_project_by_case_id(pmacontrolURL, pmacontrolCaseID, pmacoreSessionID):
+	"""
+	Retrieve case collection based on the case ID
+	"""
+	all_colls = _pma_get_case_collections(pmacontrolURL, pmacoreSessionID)
+	for coll in all_colls:
+		for case in coll["Cases"]:
+			if case["Id"] == pmacontrolCaseID:
+				return get_project(pmacontrolURL, coll["ModuleId"], pmacoreSessionID)
+
+	return None
+	
+def get_project_by_case_collection_id(pmacontrolURL, pmacontrolCaseCollectionID, pmacoreSessionID):
+	"""
+	Retrieve case collection based on the case ID
+	"""
+	all_colls = _pma_get_case_collections(pmacontrolURL, pmacoreSessionID)
+	for coll in all_colls:
+		if coll["Id"] == pmacontrolCaseCollectionID:
+			return get_project(pmacontrolURL, coll["ModuleId"], pmacoreSessionID)
+
+	return None
+
 def search_project(pmacontrolURL, titleSubstring, pmacoreSessionID):
 	"""
 	Return the first project that has titleSubstring as part of its string; search is case insensitive
